"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const express_1 = __importDefault(require("express"));
const http_1 = __importDefault(require("http"));
const socket_io_1 = __importDefault(require("socket.io"));
const cors_1 = __importDefault(require("cors"));
const path_1 = __importDefault(require("path"));
const Players_1 = __importDefault(require("./Players"));
const Player_1 = __importDefault(require("./Player"));
const PlayerList_1 = __importDefault(require("./PlayerList"));
const Pickable_1 = __importDefault(require("./Pickable"));
const Item_1 = __importDefault(require("./Item"));
const uuid = __importStar(require("uuid"));
const fs_1 = __importDefault(require("fs"));
const minx = -3000;
const maxx = 3000;
const miny = -3000;
const maxy = 3000;
const app = express_1.default();
const server = new http_1.default.Server(app);
const io = new socket_io_1.default.Server(server);
const players = new Players_1.default();
let items = [];
const playerList = new PlayerList_1.default();
let scores = JSON.parse(fs_1.default.readFileSync(__dirname + '/../score.json').toString());
app.use(cors_1.default());
app.use(express_1.default.static(path_1.default.join(__dirname, '/../../client/static/assets')));
app.use(express_1.default.static(path_1.default.join(__dirname, '/../../client/dist')));
app.get('/', function (req, res) {
    res.sendFile('index.html', { root: __dirname + '/../../client/static/' });
});
app.get('/game', function (req, res) {
    res.sendFile('game.html', { root: __dirname + '/../../client/static/' });
});
app.get('/ded', function (req, res) {
    res.sendFile('ded.html', { root: __dirname + '/../../client/static/' });
});
server.listen(8089, function () {
    console.log('running on port 8089');
});
function makePointsItem() {
    let item = new Item_1.default('Points', "POINTS");
    let pickable = new Pickable_1.default(uuid.v4(), {
        x: Math.floor(Math.random() * (Math.abs(minx) + maxx) + minx),
        y: Math.floor(Math.random() * (Math.abs(miny) + maxy) + miny),
    }, item);
    items.push(pickable);
    io.emit('new-item', { id: pickable.id, position: pickable.position, item: { image: pickable.item.image, type: pickable.item.type } });
}
makePointsItem();
io.on('connection', function (socket) {
    console.log(socket.id, 'connected');
    socket.emit('greetings');
    socket.on('greetings', (data) => {
        let player = new Player_1.default(socket);
        player.setName(data.name);
        playerList.addPlayer(socket.id, player);
        socket.emit('your-info', player.getPlayerDescription());
        io.emit('new-player-joined', player.getPlayerDescription());
        let best = {
            name: '',
            score: 0
        };
        for (let name in scores) {
            if (scores[name] > best.score) {
                best.name = name;
                best.score = scores[name];
            }
        }
        socket.emit('best-player', best);
    });
    socket.on('get-other-players', () => {
        playerList.getPlayers().forEach((player) => {
            io.emit('new-player-joined', player.getPlayerDescription());
        });
        for (let item of items) {
            io.emit('new-item', { id: item.id, position: item.position, item: { image: item.item.image, type: item.item.type } });
        }
    });
    socket.on('disconnect', () => {
        console.log(socket.id, 'disconnected', (playerList.getPlayers().size - 1));
        playerList.removePlayer(socket.id);
        io.emit('player-disconnected', socket.id);
    });
    socket.on('key-action', (data) => {
        socket.broadcast.emit('key-action', { id: socket.id, key: data.key, isDown: data.isDown });
    });
    socket.on('player-moved', (data) => {
        let player = playerList.getPlayer(socket.id);
        if (player) {
            player.setPosition(data.position.x, data.position.y);
        }
        socket.broadcast.emit('player-moved', { id: socket.id, position: data.position });
    });
    socket.on('hit-player', (data) => {
        let player = playerList.getPlayer(data.id);
        if (player) {
            if (data.health <= 0) {
                player.socket.disconnect();
                return;
            }
            player.setHealth(data.health);
        }
        socket.broadcast.emit('hit-player', { id: data.id, health: data.health });
    });
    socket.on('shoot', (data) => {
        socket.broadcast.emit('shoot', { id: socket.id, position: data.position, direction: data.direction });
    });
    socket.on('item-picked', (data) => {
        console.log('item picked');
        let player = playerList.getPlayer(socket.id);
        if (player) {
            console.log('player found');
            for (let item of items) {
                if (item.id === data.id && item.item.type === "POINTS") {
                    console.log('item points');
                    player.addScore();
                    if (scores[player.getName()] && scores[player.getName()] < player.getScore()) {
                        console.log('update score');
                        scores[player.getName()] = player.getScore();
                        fs_1.default.writeFileSync(__dirname + '/../score.json', JSON.stringify(scores));
                    }
                    else {
                        console.log('new score');
                        scores[player.getName()] = player.getScore();
                        fs_1.default.writeFileSync(__dirname + '/../score.json', JSON.stringify(scores));
                    }
                    let best = {
                        name: '',
                        score: 0
                    };
                    for (let name in scores) {
                        if (scores[name] > best.score) {
                            best.name = name;
                            best.score = scores[name];
                        }
                    }
                    io.emit('best-player', best);
                    io.emit('update-score', { id: player.socket.id, score: player.getScore() });
                    makePointsItem();
                    break;
                }
            }
        }
        items = items.filter((item) => {
            return item.id !== data.id;
        });
        socket.broadcast.emit('item-picked-resolve', { id: data.id });
    });
});
setInterval(() => {
    if (items.length < 21) {
        let item = new Item_1.default('healthPack', "HEAL");
        let pickable = new Pickable_1.default(uuid.v4(), {
            x: Math.floor(Math.random() * (Math.abs(minx) + maxx) + minx),
            y: Math.floor(Math.random() * (Math.abs(miny) + maxy) + miny),
        }, item);
        items.push(pickable);
        io.emit('new-item', { id: pickable.id, position: pickable.position, item: { image: pickable.item.image, type: pickable.item.type } });
    }
}, 15000);
//# sourceMappingURL=app.js.map